@@ -4,6 +4,7 @@ import binascii
 import collections
 import hashlib
 import itertools
+import logging
 import random
 import string
 import time
@@ -15,6 +16,8 @@ import msgpack
 from channels.exceptions import ChannelFull
 from channels.layers import BaseChannelLayer
 
+logger = logging.getLogger(__name__)
+
 
 def _wrap_close(loop, pool):
     """
@@ -69,6 +72,9 @@ class ConnectionPool:
         if not conns:
             conns.append(await aioredis.create_redis(**self.host, loop=loop))
         conn = conns.pop()
+        if conn.closed:
+            conn = await self.pop(loop=loop)
+            return conn
         self.in_use[conn] = loop
         return conn
 
@@ -615,9 +621,11 @@ class RedisChannelLayer(BaseChannelLayer):
                 x.decode("utf8") for x in await connection.zrange(key, 0, -1)
             ]
 
-        connection_to_channel_keys, channel_keys_to_message, channel_keys_to_capacity = self._map_channel_keys_to_connection(
-            channel_names, message
-        )
+        (
+            connection_to_channel_keys,
+            channel_keys_to_message,
+            channel_keys_to_capacity,
+        ) = self._map_channel_keys_to_connection(channel_names, message)
 
         for connection_index, channel_redis_keys in connection_to_channel_keys.items():
 
@@ -627,13 +635,16 @@ class RedisChannelLayer(BaseChannelLayer):
             # __asgi_channel__ key.
 
             group_send_lua = (
-                """
+                """ local over_capacity = 0
                     for i=1,#KEYS do
                         if redis.call('LLEN', KEYS[i]) < tonumber(ARGV[i + #KEYS]) then
                             redis.call('LPUSH', KEYS[i], ARGV[i])
                             redis.call('EXPIRE', KEYS[i], %d)
+                        else
+                            over_capacity = over_capacity + 1
                         end
                     end
+                    return over_capacity
                     """
                 % self.expiry
             )
@@ -652,9 +663,13 @@ class RedisChannelLayer(BaseChannelLayer):
 
             # channel_keys does not contain a single redis key more than once
             async with self.connection(connection_index) as connection:
-                await connection.eval(
+                channels_over_capacity = await connection.eval(
                     group_send_lua, keys=channel_redis_keys, args=args
                 )
+                if channels_over_capacity > 0:
+                    logger.exception(
+                        f"{channels_over_capacity} of {len(channel_names)} channels over capacity in group {group}"
+                    )
 
     def _map_channel_to_connection(self, channel_names, message):
         """
